(function() {
	var mraid = window.mraid = window.ormma = {};

	var STATES = mraid.STATES = {
		LOADING		: 'loading',
		DEFAULT		: 'default',
		RESIZED		: 'resized',
		EXPANDED	: 'expanded',
		HIDDEN		: 'hidden'
	};

	var EVENTS = mraid.EVENTS = {
		ERROR				: 'error',
		READY				: 'ready',
		SIZECHANGE			: 'sizeChange',
		STATECHANGE			: 'stateChange',
		VIEWABLECHANGE		: 'viewableChange'
	};

	var FEATURES = mraid.FEATURES = {
		SMS			: 'sms',
		TEL			: 'tel',
		CALENDAR	: 'calendar',
		PICTURE		: 'storePicture',
		INLINEVIDEO	: 'inlineVideo'
	};

	// PRIVATE PROPERTIES (sdk controlled) //////////////////////////////////////////////////////

	var currentState = STATES.LOADING;

	var currentExpandProperties = null;

	var currentOrientationProperties = null;

	var currentResizeProperties = null;

	var currentSupports = {
		'sms'			: false,
		'tel'			: false,
		'calendar'		: false,
		'storePicture'	: false,
		'inlinevideo'	: false
	};

	// PRIVATE PROPERTIES (internal) //////////////////////////////////////////////////////

	var changeHandlers = {
		onInitialSupports : function(val) {
			currentSupports = val;
		},
		onInitialExpandProperties : function(val) {
			currentExpandProperties = val;
		},
		onInitialOrientationProperties : function(val) {
			currentOrientationProperties = val;
		},
		onInitialResizeProperties : function(val) {
			currentResizeProperties = val;
		},
		onInitialState : function(val) {
			currentState = val;
		},
		onReady : function(val) {
			broadcastEvent(EVENTS.READY);
		},
		onSize : function(val) {
			broadcastEvent(EVENTS.SIZECHANGE, val.width, val.height);
		},
		onState : function(val) {
			currentState = val;
			broadcastEvent(EVENTS.STATECHANGE, currentState);
		},
		onViewable : function(val) {
			broadcastEvent(EVENTS.VIEWABLECHANGE, val);
		}
	};

	var listeners = {};

	var EventListeners = function(event) {
		this.event = event;
		this.count = 0;
		var listeners = {};

		this.add = function(func) {
			var id = String(func);
			if (!listeners[id]) {
				listeners[id] = func;
				this.count++;
			}
		};
		this.remove = function(func) {
			var id = String(func);
			if (listeners[id]) {
				listeners[id] = null;
				delete listeners[id];
				this.count--;
				return true;
			} else {
				return false;
			}
		};
		this.removeAll = function() {
			for (var id in listeners) {
				this.remove(listeners[id]);
			}
		};
		this.broadcast = function(args) {
			for (var id in listeners) {
				listeners[id].apply({}, args);
			}
		};
		this.toString = function() {
			var out = [ event, ':' ];
			for (var id in listeners) {
				out.push('|', id, '|');
			}
			return out.join('');
		};
	};

	var clone = function(obj) {
		var f = function() {
		};
		f.prototype = obj;
		return new f();
	};
	
	var parse = function(args) {
		if (typeof JSON === "undefined") {
			return eval('(' + args	+ ')');
		} else {
			return JSON.parse(args);
		}
	}

	stringify = function(args) {
		if (typeof JSON === "undefined") {
			var s = "";
			var len = args.length;
			var i;
			if (typeof len == "undefined") {
				return stringifyArg(args);
			}
			for (i = 0; i < args.length; i++) {
				if (i > 0) {
					s = s + ",";
				}
				s = s + stringifyArg(args[i]);
			}
			s = s + "]";
			return s;
		} else {
			return JSON.stringify(args);
		}
	};

	stringifyArg = function(arg) {
		var s, type, start, name, nameType, a;
		type = typeof arg;
		s = "";
		if ((type === "number") || (type === "boolean")) {
			s = s + args;
		} else if (arg instanceof Array) {
			s = s + "[" + arg + "]";
		} else if (arg instanceof Object) {
			start = true;
			s = s + '{';
			for (name in arg) {
				if (arg[name] !== null) {
					if (!start) {
						s = s + ',';
					}
					s = s + '"' + name + '":';
					nameType = typeof arg[name];
					if ((nameType === "number") || (nameType === "boolean")) {
						s = s + arg[name];
					} else if ((typeof arg[name]) === 'function') {
						// don't copy the functions
						s = s + '""';
					} else if (arg[name] instanceof Object) {
						s = s + this.stringify(args[i][name]);
					} else {
						s = s + '"' + arg[name] + '"';
					}
					start = false;
				}
			}
			s = s + '}';
		} else {
			a = arg.replace(/\\/g, '\\\\');
			a = a.replace(/"/g, '\\"');
			s = s + '"' + a + '"';
		}
		
		return s;
	};

	var contains = function(value, array) {
		for (var i in array) {
			if (array[i] == value) {
				return true;
			}
		}
		return false;
	};

	var broadcastEvent = function() {
		var args = new Array(arguments.length);
		for (var i = 0; i < arguments.length; i++) {
			args[i] = arguments[i];
		}
		var event = args.shift();
		try {
			if (listeners[event]) {
				listeners[event].broadcast(args);
			}
		} catch (e) {
		}
	};

	var trim = function(s) {
		var l = 0;
		var r = s.length - 1;
		while (l < s.length && s[l] == ' ') {
			l++;
		}
		while (r > l && s[r] == ' ') {
			r -= 1;
		}
		return s.substring(l, r + 1);
	};

	mraid.addEventListener = function(event, listener) {
		if (!event || !listener) {
			broadcastEvent(EVENTS.ERROR, 'Both event and listener are required.', 'addEventListener');
		} else if (!contains(event, EVENTS)) {
			broadcastEvent(EVENTS.ERROR, 'Unknown event: ' + event, 'addEventListener');
		} else {
			if (!listeners[event]) {
				listeners[event] = new EventListeners(event);
			}
			listeners[event].add(listener);
		}
	};

	mraid.close = function() {
		try {
			MraidController.close();
		} catch (e) {
			MraidController.showAlert("close: " + e);
		}
	};

	mraid.createCalendarEvent = function(val) {
		var description = val.description;
		var location = val.location;
		var summary = val.summary;
		var start = new Date(val.start);
		var end = new Date(val.end);

		if (!currentSupports[FEATURES.CALENDAR]) {
			broadcastEvent(EVENTS.ERROR, 'Method not supported by this client.', 'createCalendarEvent');
		} else {
			try {
				MraidController.createCalendarEvent(description, location, summary, start.getTime().toString(), end.getTime().toString());
			} catch (e) {
				MraidController.showAlert("createCalendarEvent: " + e);
			}
		}
	};

	mraid.expand = function(url) {
		try {
			MraidController.expand(url, stringify(currentExpandProperties), stringify(currentOrientationProperties));
		} catch (e) {
			MraidController.showAlert("expand: " + e);
		}
	};

	mraid.getCurrentPosition = function() {
		return parse(MraidController.getCurrentPosition());
	};

	mraid.getDefaultPosition = function() {
		return parse(MraidController.getDefaultPosition());
	};

	mraid.getExpandProperties = function() {
		return currentExpandProperties == null ? null : clone(currentExpandProperties);
	};

	mraid.getMaxSize = function() {
		return parse(MraidController.getMaxSize());
	};

	mraid.getOrientationProperties = function() {
		return currentOrientationProperties == null ? null : clone(currentOrientationProperties);
	};

	mraid.getPlacementType = function() {
		return MraidController.getPlacementType();
	};

	mraid.getResizeProperties = function() {
		return currentResizeProperties == null ? null : clone(currentResizeProperties);
	};

	mraid.getScreenSize = function() {
		return parse(MraidController.getScreenSize());
	};

	mraid.getState = function() {
		return currentState;
	};

	mraid.getVersion = function() {
		return "2.0";
	};

	mraid.isViewable = function() {
		return MraidController.isViewable();
	};

	mraid.open = function(url) {
		if (!url) {
			broadcastEvent(EVENTS.ERROR, 'url is required.', 'open');
		} else {
			try {
				MraidController.open(url);
			} catch (e) {
				MraidController.showAlert("open: " + e);
			}
		}
	};

	mraid.playVideo = function(url) {
		if (!url || typeof url != 'string') {
			broadcastEvent(EVENTS.ERROR, 'Request must specify a url', 'playVideo');
		} else {
			try {
				MraidController.playVideo(url);
			} catch (e) {
				MraidController.showAlert("playVideo: " + e);
			}
		}
	};

	mraid.removeEventListener = function(event, listener) {
		if (!event) {
			broadcastEvent(EVENTS.ERROR, 'Must specify an event.', 'removeEventListener');
		} else {
			if (listener
					&& (!listeners[event] || !listeners[event].remove(listener))) {
				broadcastEvent(EVENTS.ERROR,
						'Listener not currently registered for event', 'removeEventListener');
				return;
			} else if (listeners[event]) {
				listeners[event].removeAll();
			}

			if (listeners[event] && listeners[event].count == 0) {
				listeners[event] = null;
				delete listeners[event];
			}
		}
	};

	mraid.resize = function() {
		try {
			MraidController.resize(stringify(currentResizeProperties));
		} catch (e) {
			MraidController.showAlert("resize: " + e);
		}
	};

	mraid.setExpandProperties = function(properties) {
		var fields = [ 'width', 'height', 'useCustomClose', 'isModal' ];

		for (var f in fields) {
			var field = fields[f];
			if (properties[field] !== undefined) {
				currentExpandProperties[field] = properties[field];
			}
		}
	};

	mraid.setOrientationProperties = function(properties) {
		var fields = [ 'allowOrientationChange', 'forceOrientation' ];

		for (var f in fields) {
			var field = fields[f];
			if (properties[field] !== undefined) {
				currentOrientationProperties[field] = properties[field];
			}
		}

		MraidController.setOrientationProperties(stringify(currentOrientationProperties));
	};

	mraid.setResizeProperties = function(properties) {
		var fields = [ 'width', 'height', 'customClosePosition', 'offsetX', 'offsetY', 'allowOffscreen' ];

		for (var f in fields) {
			var field = fields[f];
			if (properties[field] !== undefined) {
				currentResizeProperties[field] = properties[field];
			}
		}
	};

	mraid.storePicture = function(url) {
		if (!currentSupports[FEATURES.PICTURE]) {
			broadcastEvent(EVENTS.ERROR, 'Method not supported by this client.', 'storePicture');
		} else {
			MraidController.storePicture(url);
		}
	};

	mraid.supports = function(feature) {
		if (currentSupports[feature]) {
			return true;
		} else {
			return false;
		}
	};

	mraid.useCustomClose = function(flag) {
		mraid.setExpandProperties({
			"useCustomClose" : flag
		});

		MraidController.useCustomClose(flag);
	};

	var mraidview = window.mraidview = {};

	var bridge_listeners = {};

	mraidview.fireChangeEvent = function(properties) {
		var handlers = bridge_listeners['change'];
		if (handlers != null) {
			for (var i = 0; i < handlers.length; i++) {
				handlers[i](properties);
			}
		}

		return "OK";
	};

	mraidview.fireErrorEvent = function(message, action) {
		var handlers = bridge_listeners['error'];
		if (handlers != null) {
			for (var i = 0; i < handlers.length; i++) {
				handlers[i](message, action);
			}
		}

		return "OK";
	};

	mraidview.addEventListener = function(event, listener) {
		var handlers = bridge_listeners[event];
		if (handlers == null) {
			// no handlers defined yet, set it up
			bridge_listeners[event] = [];
			handlers = bridge_listeners[event];
		}

		// see if the listener is already present
		for (var handler in handlers) {
			if (listener == handler) {
				// listener already present, nothing to do
				return;
			}
		}

		// not present yet, go ahead and add it
		handlers.push(listener);
	};

	mraidview.removeEventListener = function(event, listener) {
		var handlers = bridge_listeners[event];
		if (handlers != null) {
			handlers.remove(listener);
		}
	};

	mraidview.addEventListener('change', function(properties) {
		for (var property in properties) {
			var handler = changeHandlers[property];
			handler(properties[property]);
		}
	});

	mraidview.addEventListener('error', function(message, action) {
		broadcastEvent(EVENTS.ERROR, message, action);
	});
})();

(function() {
	var asum = window.asum = {};

	asum.sendEventUrl = function(url) {
		AsumController.sendEventUrl(url);
	};

	asum.openOptoutUrl = function(url) {
		AsumController.openOptoutUrl(url);
	};
})();