/*!
* 1. 사이트 설정
* 2. 사이트 전반에 대한 기본처리
**/
vcui.require.config({
    paths: {
        'jquery.transit': 'libs/jquery.transit',
        'Raphael': 'libs/raphael.min'
    },
    waitSeconds: 15
});

var site = {
    isLocal: window.location.hostname === 'localhost',
    isDev: window.location.hostname.indexOf('jsdebug=true') > -1,
    googleMapsAPIKey: '',
    /**
     * 모달이 열렸을 때 페이지 스크롤은 막는다.
     * @param flag
     */
    preventPageScroll: function (flag) {
        var $wrap = $('#wrap');

        if (flag) {
            var scrollTop = $(window).scrollTop();
            $wrap.css({'position': 'fixed', 'top': -scrollTop});
        } else {
            var top = parseInt($wrap.css('top'), 10) || 0;
            $wrap.css({'position': '', 'top': ''});
            $('html, body').scrollTop(-top);
        }
    },
    // 사이트 기본작업들 처리
    init: function () {
        this._preloadComponents();
        this._initInputNumberMaxlength();
    },
    // 주요 컴포넌트를 미리 로드
    _preloadComponents: function () {
        var self = this;

        $.holdReady(true); // ready 실행을 잠시 멈춘다.
        vcui.require([
            'ui/modal'
        ], function () {
            vcui.ui.setDefaults('Modal', {
                useTransformAlign: false
            });
            self._init();
            $.holdReady(false); // ready함수 실행을 허용(이전에 등록된건 실행해준다.)
        });
    },
    _init: function () {
        // 모달 기본옵션 설정: 모달이 들때 아무런 모션도 없도록 한다.(기본은 fade)
        vcui.ui.setDefaults('Modal', {
            effect: 'none'
        });

        // 모달이 열렸을 때 페이지 스크롤을 막기 위함
        $(document).on('modalfirstopen modallastclose', function (e) {
            site.preventPageScroll(e.type === 'modalfirstopen');
            $('#wrap').attr('aria-hidden', e.type === 'modalfirstopen' ? 'true' : 'false');
        }).on('modalshown', function (e) {
            $(e.target).buildCommonUI();
        });

        // 공통 UI 빌드
        $('body').buildCommonUI();

        // ios에서 키보드가 올라올 때 밑의 fixed 버튼을 안보이게 처리
        if (vcui.detect.isIOS) {
            $(document).on('focusin focusout', 'input, textarea, select, [contenteditable]', function (e) {
                $('body').removeClass('hidden_fixed');

                if (this.readOnly) { return; }
                if (e.type === 'focusin') {
                    if (this.type === 'checkbox' || this.type === 'radio') {
                        return;
                    }
                    $('body').addClass('hidden_fixed');
                }
            });
        }
    },
    // input type=number에는 maxlength 가 안먹어서 스크립트로 처리
    _initInputNumberMaxlength: function () {
        $(document).on('input.inputnumbermaxlength', '[type=number][maxlength]', function (e) {
            var maxLength = this.maxLength || 0;
            if (!maxLength) {
                return;
            }
            this.value = this.value.substr(0, maxLength);
        })
    }
};

// 초기작업 실행
site.init();

// 페이지가 브라우저에 의해 캐시되었는지 여부에 체크하여 새로고침을 해준다.
window.onpageshow = function (event) {
    if (event.persisted) {
        window.location.reload();
    }
};
