/*!
 * @module vcui.ui.BarChart
 * @license MIT License
 * @description LineChart 컴포넌트
 * @copyright VinylC UID Group
 * 
 * 
 */
define('ui/barChart', ['jquery', 'vcui', 'libs/raphael.min'], function ($, core, Raphael) {
    "use strict";

    /**
     * @class
     * @description
     * @name
     * @extends vcui.ui.View
     */

    var BarChart = core.ui('BarChart', /** @lends vcui.ui.BarChart# */ {
        bindjQuery: 'barChart',
        defaults: {
            //dataArr: [],
            //stageWidth: null,
            //stageHeight: null,
            sizeRadio: 0.7,
            margin: {
                top: 70,
                right: 20,
                bottom: 50,
                left: 20
            },
            minBarHeight:2,
            pointSize: 20,
            isAnimate: false,
            isAverage: false,
            keyClasName : 'fl_l',
            valueClasName : 'fl_r',
            unit: "원" //mmss

        },
        selectors: {
        },

        initialize: function initialize(el, options) {
            var self = this;
            if (self.supr(el, options) === false) {
                return;
            }

            //console.log(self.$el.find('svg').length);            
            //if(self.$el.find('svg').length>0) return;

            Raphael.el.addClass = function (className) {
                this.node.setAttribute("class", className);
                return this;
            };

            
            self._setting();
            self._build();
        },


        _setting: function _setting() {
            var self = this,
                opts = self.options;

            self.stageWidth = opts.stageWidth || self.$el.width();
            self.stageHeight = opts.stageHeight || self.stageWidth * opts.sizeRadio;
            self.margin = opts.margin;


            self.drawHeight = self.stageHeight - (self.margin.top + self.margin.bottom);
            self.drawWidth = self.stageWidth - (self.margin.left + self.margin.right);
            self.pointSize = opts.pointSize;
            self.isAverage = opts.isAverage;
            self.unit = opts.unit;

            self.minBarHeight = opts.minBarHeight;

            var dataArr = [];

            if (opts.target) {

                var obj, val;

                $(opts.target).children().each(function (idx, target) {

                    var key = $(target).find('.'+opts.keyClasName).text();
                    var keyHide = $(target).find('.'+opts.keyClasName+' .hide').text();
                    var value = $(target).find('.'+opts.valueClasName).text();
                    var valueHide = $(target).find('.'+opts.valueClasName + ' .hide').text();                    

                    if (keyHide) {
                        key = key.replace(keyHide, "");
                    }

                    if (valueHide) {
                        value = value.replace(valueHide, "");
                    }

                    if ($(target).hasClass('ui_average_txt')) {                        
                        return;
                    }

                    if (self.unit == "mmss") {
                        var vArr = value.replace("분", "/").split("/");
                        if (vArr.length > 1) {
                            val = core.number.parse(vArr[0]) * 60 + core.number.parse(vArr[1]);
                        }else{                            
                            val = value.replace(/\D/g, '');
                        }
                    } else {
                        val = core.number.parse(value);                        
                    }
                    
                    if(val<0) val = 0;

                    var keyTilte =  $(target).find('.'+opts.keyClasName).data('title');
                    var valueTilte =  $(target).find('.'+opts.valueClasName).data('title');

                    obj = {
                        'title': keyTilte? keyTilte : key,
                        'label': valueTilte? valueTilte : value,
                        'value': val
                    };
                    dataArr.push(obj);
                });

                //console.log(dataArr);


            }


            self.dataArr = opts.dataArr || dataArr;
            self.aniArr = [];
            self.animArr = [];



        },

        _build: function _build() {

            var self = this;

            self.paper = Raphael(0, 0, "100%", "100%");
            self.paper.setViewBox(0, 0, self.stageWidth, self.stageHeight, false);
            self.$el.append(self.paper.canvas);
            self._createBar();
        },

        _createBar: function _createBar() {

            var self = this,
                opts = self.options;

            var minVal = parseFloat(Math.min.apply(Math, self.dataArr.map(function (o) {
                return o.value;
            })));
            var maxVal = parseFloat(Math.max.apply(Math, self.dataArr.map(function (o) {
                return o.value;
            })));   
            
            if(opts.max){
                if (self.unit == "mmss") {
                    var vArr = opts.max.replace("분", "/").split("/");
                    if (vArr.length > 1) {
                        maxVal = core.number.parse(vArr[0]) * 60 + core.number.parse(vArr[1]);
                    }else{                            
                        maxVal = opts.max.replace(/\D/g, '');
                    }
                } else {
                    maxVal = core.number.parse(opts.max);                        
                }
            }
            

            var gRatio = (self.drawHeight-self.minBarHeight) / maxVal; //(self.drawHeight-self.minBarHeight) / (maxVal - minVal);
            var gDistance = self.drawWidth / self.dataArr.length;
            var fx, ex, txt, xp, yp, obj, label, bar;
            var drawTop = self.drawHeight + self.margin.top;           


            for (var i = 0; i < self.dataArr.length; i++) {

                obj = self.dataArr[i];

                if(gRatio == Infinity){
                    yp = self.minBarHeight;
                }else{
                    yp = self.drawHeight - (maxVal - parseFloat(obj.value)) * gRatio + self.minBarHeight;
                }

                xp = gDistance * i + self.margin.left + (gDistance-self.pointSize)/2;

                txt = self.paper.text(xp+(self.pointSize/2), drawTop + 25, obj.title).attr({
                    "font-size": 12
                }).addClass("chart_title");
                
                fx = xp + (self.pointSize/2) - txt.getBBox().width/2;
                ex = xp + (self.pointSize/2) + txt.getBBox().width/2;       
                

                if (fx < 0) txt.attr({x : txt.getBBox().width/2});
                if (ex > self.stageWidth) txt.attr({x : self.stageWidth - txt.getBBox().width/2});


                self.paper.rect(xp, self.margin.top, self.pointSize, self.drawHeight).attr({
                    "fill": "#EEEEEE",
                    "stroke": "none"
                });
                
                bar = self.paper.rect(xp, drawTop, self.pointSize, 1).attr({
                    "fill": "#E71A45",
                    "stroke": "none"
                });


                label = self.paper.text(xp+(self.pointSize/2), self.margin.top-self.pointSize,  obj.label).attr({
                    "font-size": 12,
                    "opacity":0,
                    "transform":"t0,20"
                }).addClass("chart_label");


                if (fx < 0) label.attr({x : label.getBBox().width/2});
                if (ex > self.stageWidth) label.attr({x : self.stageWidth - label.getBBox().width/2});


                if (!opts.isAnimate) {
                    bar.attr({"y":(drawTop-yp),"height":yp});
                    label.attr({"opacity":1, "transform":"t0,0"});
                }

                
                self.aniArr.push({
                    target: bar,
                    param: {
                        "y" : (drawTop-yp),
                        "height" : yp
                    }
                });

                self.aniArr.push({
                    target: label,
                    param: {
                        "opacity" : 1,
                        "transform":"t0,0"
                    },
                    duration: 300,
                    delay: parseInt(yp)*5
                });

            }

            self.paper.path(["M", self.margin.left, drawTop, "L", self.drawWidth+self.margin.left, drawTop]).attr({
                "fill": 'none',
                "stroke": '#888888',
                "stroke-width": 1
            });

        },


        goAnimate: function goAnimate() {

            var self = this,
                opts = self.options;

            if (!opts.isAnimate) return;

            for (var i = 0; i < self.aniArr.length; i++) {
                var obj = self.aniArr[i];
                if (obj.delay) {
                    var anim = Raphael.animation(obj.param, obj.duration, '<>');
                    obj["target"].animate(anim.delay(obj.delay));
                    self.animArr.push(anim);
                } else {
                    obj["target"].animate(obj["param"], 1000, '<>');
                }
            }

        },


        clear: function clear() {

            var self = this;
            
            for (var i = 0; i < self.aniArr.length; i++) {
                self.aniArr[i]["target"].stop();
            }

            for (var i = 0; i < self.animArr.length; i++) {
                self.animArr[i].stop();
            }

            self.aniArr = [];
            self.animArr = [];
            self.paper.clear();
        },
        /**
         * 이벤트 바인딩
         * @private
         */
        _bindEvents: function _bindEvents() {
            var self = this;
        }

    });
    ///////////////////////////////////////////////////////////////////////////////////////

    return BarChart;
});