/*!
 * @module vcui.ui.FakeFreezeHeaders
 * @license MIT License
 * @description FreezeHeader 컴포넌트
 * @copyright VinylC UID Group
 */
define('ui/fakeFreezeHeaders', ['jquery', 'vcui'], function ($, core) {
    "use strict";

    function offsetX(el) {
        var left = 0,
            attr = 'offsetParent';

        if (el[attr]) {
            while (1) {
                left += el.offsetLeft;
                if (!el[attr]) {
                    break;
                }
                el = el[attr];
            }
        } else if (el.x) {
            left += el.x;
        }
        return left;
    }

    function offsetY(el) {
        var top = 0,
            attr = 'offsetParent';
        if (el[attr]) {
            while (1) {
                top += el.offsetTop;
                if (!el[attr]) {
                    break;
                }
                el = el[attr];
            }
        } else if (el.y) {
            top += el.y;
        }
        return top;
    }

    var FakeFreezeHeaders = core.ui('FakeFreezeHeaders', {
        bindjQuery: 'FakeFreezeHeaders',
        defaults: {
            //header,
            offsetTop: 0,
            offsetBottom: 0,
            fixHeader: '',
            headerSelector: 'dt>div',
            scrollWrapper: 'body',
            // 고정대상 여부를 판단
            allowFreeze: function () {
                return true;
            }
        },
        initialize: function initialize(el, options) {
            var self = this;

            if (self.supr(el, options) === false) {
                return;
            }

            // 헤더들 추출
            self.$headers = self.$(self.options.headerSelector);
            // 싫제 고정되는 헤더요소
            self.$fixHeader = $(self.options.fixHeader);
            self.$fixedTopHeader = $('.top_head_wrap').css('-webkit-backface-visibility', 'hidden');
            self.fixedTopHeaderHeight = core.array.reduce(self.$fixedTopHeader.find('>*:first').get(), function (sum, el) {
                return sum + $(el).outerHeight();
            }, 0);

            self._bindEvents();
        },
        /**
         * 외부호출용 갱신 함수
         */
        update: function update() {
            var self = this;

            self.scrollTop = window.pageYOffset; // $win.scrollTop();
            self._position(true);
        },
        /**
         * 이벤트 바인딩
         * @private
         */
        _bindEvents: function _bindEvents() {
            var self = this,
                opt = self.options;

            self.winOn('scroll resize load', function () {
                self.scrollTop = window.pageYOffset; // $win.scrollTop();
                self._position();
            }); //

        },

        isOutOfBounds: function (currentScrollY) {
            var pastTop = currentScrollY < 0,
                pastBottom = currentScrollY + core.dom.getWinHeight() > core.dom.getDocHeight();

            return pastTop || pastBottom;
        },

        /**
         * 고정헤더 숨김
         * @private
         */
        _hide: function _hide() {
            var self = this;

            if (self.isShownHeader) {
                self.isShownHeader = false;
                self.$fixHeader.parent().hide();//.stop(true).fadeOut('fast');
            }
            self.activeIndex = -1;
        },
        /**
         * 고정헤더 표시
         * @private
         */
        _show: function () {
            var self = this;

            if (!self.isShownHeader) {
                self.isShownHeader = true;
                self.$fixHeader.parent().show(); //.stop(true).fadeIn('fast');
            }
        },
        /**
         * 각 헤더의 위치를 추적하여 해당하는 제목으로 교체하고 헤더에 고정시켜 준다.
         * @private
         */
        _position: function _position(isForce) {
            if (this.isOutOfBounds(this.scrollTop)) {
                return;
            }

            var self = this,
                currentScrollY = self.scrollTop,
                rect = core.dom.getRect(self.el),
                dir = currentScrollY > self.lastScrollY ? 'up' : 'down',
                diff = (dir === 'down' ? self.fixedTopHeaderHeight : 0),
                relativeScrollTop = self.scrollTop + diff;

            if (isForce !== true && currentScrollY > 0 && self.lastScrollY === currentScrollY) {
                return;
            }

            if (currentScrollY < self.fixedTopHeaderHeight) {
                if (dir === 'up' || !self._shownHeader) {
                    self._moveY(-currentScrollY, false);
                    self._shownHeader = false;
                }
            } else {
                if (dir === 'up') {
                    //if (self._shownHeader) {
                        self._moveY(-100);
                        self._shownHeader = false;
                    //}
                } else {
                    //if (!self._shownHeader) {
                        self._moveY(0);
                        self._shownHeader = true;
                    //}
                }
            }

            self.lastScrollY = currentScrollY;

            if (relativeScrollTop < rect.top) {
                self._hide();
                return;
            }

            self._fixedHeader();
        },

        _fixedHeader: function () {
            var self = this,
                opt = self.options,
                currentScrollY = self.scrollTop,
                nextHeader = void 0,
                height = void 0,
                headerTop = void 0,
                nextHeaderTop = void 0,
                $fixedTopHeaderTitle = self.$fixHeader.find('.ui_freezeHeader_title'),
                dir = currentScrollY > self.lastScrollY ? 'up' : 'down',
                diff = (dir === 'down' ? self.fixedTopHeaderHeight : 0),
                relativeScrollTop = self.scrollTop + diff;


            for (var i = -1, el; el = self.$headers[++i];) {
                headerTop = offsetY(el);

                if (relativeScrollTop > headerTop) {
                    height = el.offsetHeight;
                    nextHeader = self.$headers[i + 1];
                    nextHeaderTop = (nextHeader ? offsetY(nextHeader) : 1000000) - height;//rect.bottom;

                    if (relativeScrollTop - height < nextHeaderTop/* && self.activeIndex !== i*/) {
                        if (!opt.allowFreeze(el) || relativeScrollTop > nextHeaderTop) {
                            self._hide();
                            //break;
                        } else {
                            self._show();
                            self.activeIndex = i; // 현재 활성화된 헤더의 인덱스 번호
                            $fixedTopHeaderTitle.text($(el).find('.ui_freezeHeader_title').text()).parent().data('index', i);
                        }
                    }
                }
            }
        },

        _moveY: function (y, isAni) {
            var self = this;

            self.$fixedTopHeader.css({
                'transition': isAni === false ? 'none' : 'transform 0.3s',
                'transform': 'translate(0, ' + y + 'px)'
            });
        }
    });

    return FakeFreezeHeaders;
});