/*!
 * @module vcui.ui.FreezeHeaders
 * @license MIT License
 * @description FreezeHeader 컴포넌트
 * @copyright VinylC UID Group
 */
define('ui/freezeHeaders', ['jquery', 'vcui'], function ($, core) {
    "use strict";

    function offsetX(el) {
        var left = 0,
            attr = 'offsetParent';

        if (el[attr]) {
            while (1) {
                left += el.offsetLeft;
                if (!el[attr]) {
                    break;
                }
                el = el[attr];
            }
        } else if (el.x) {
            left += el.x;
        }
        return left;
    }

    function offsetY(el) {
        var top = 0,
            attr = 'offsetParent';
        if (el[attr]) {
            while (1) {
                top += el.offsetTop;
                if (!el[attr]) {
                    break;
                }
                el = el[attr];
            }
        } else if (el.y) {
            top += el.y;
        }
        return top;
    }

    var FreezeHeaders = core.ui('FreezeHeaders', {
        bindjQuery: 'FreezeHeaders',
        defaults: {
            //header,
            offsetTop: 0,
            headerSelector: 'dt>div',
            scrollWrapper: 'body'
        },
        initialize: function initialize(el, options) {
            var self = this;

            if (self.supr(el, options) === false) {
                return;
            }

            self.$wrapper = $(self.options.scrollWrapper);
            self.$headers = self.$(self.options.headerSelector);
            self.$fixedHeader = $('.top_head');

            self.build();
            self._bindEvents();
        },
        build: function build() {
            var self = this;

            var tag = self.$headers[0].tagName;
            self.headers = self.$headers.map(function () {
                var $holder = $(this).wrap('<' + tag + ' style="width:100%;height:' + $(this).outerHeight() + 'px" class="ui-freezeheaders-holder"></' + tag + '>').parent();

                return {
                    el: this,
                    holder: $holder[0]
                };
            }).get();
        },
        update: function update() {
            var self = this;

            self.headers = self.$headers.map(function () {
                return {
                    el: this,
                    holder: $(this).prev()[0]
                };
            }).get();
        },
        _bindEvents: function _bindEvents() {
            var self = this;

            self.winOn('scroll resize load', function () {
                self.scrollTop = window.pageYOffset; // $win.scrollTop();
                self._position();
            });
        },
        _position: function _position() {
            var self = this;
            var rect = core.dom.getDimensions(self.el);
            var currentScrollY = self.scrollTop;
            var wrapperBottom = rect.bottom + currentScrollY;

            var el = void 0,
                holder = void 0,
                nextHeader = void 0,
                height = void 0,
                holderTop = void 0,
                position = void 0,
                top = void 0,
                zIndex = void 0;

            var dir = currentScrollY > self.lastScrollY ? 'up' : 'down';
            var diff = (dir === 'down' ? 100 : 0);
            var ttop = currentScrollY + diff;

            self.headerHeight = 100;

            if (currentScrollY > 0 && self.lastScrollY === currentScrollY) {
                return;
            }


            for (var i = -1, item; item = self.headers[++i];) {
                el = item.el;
                height = el.offsetHeight;
                holder = item.holder;
                holderTop = offsetY(holder);
                nextHeader = self.headers[i + 1];
                position = 'fixed';

                if (self.scrollTop > holderTop) {
                    if (nextHeader) {
                        var diff = offsetY(nextHeader.el) - currentScrollY;

                        if (diff < height) {
                            top = -(height - diff);
                        } else {
                            top = 0;
                        }
                    } else {
                        if (wrapperBottom - height < currentScrollY) {
                            top = -(currentScrollY - wrapperBottom + height);
                        } else {
                            top = 0;
                        }
                    }

                    top += 98;
                    top += 'px';
                    zIndex = 9999;
                } else {
                    position = '';
                    top = '';
                    zIndex = '';
                }

                el.style.position = position;
                el.style.top = top;
                el.style.left = el.style.right = 0;
                el.style.zIndex = zIndex;
            }


            if (currentScrollY > self.lastScrollY && currentScrollY < self.headerHeight) {
                self.$fixedHeader.css({'transition': 'none', 'transform': 'translate(0, -' + currentScrollY + 'px)'});
            } else {
                if (currentScrollY > self.lastScrollY) {
                    //log('up');
                    self.$fixedHeader.css({'transition': 'transform 0.3s', 'transform': 'translate(0, -100px)'});
                } else {
                    //log('down');
                    self.$fixedHeader.css({'transition': 'transform 0.3s', 'transform': 'translate(0, 0px)'});
                }
            }

            self.lastScrollY = currentScrollY;
        },

        _getHeaderTop:  function () {
            var self = this;

            return core.css3.getTranslateXY(self.$fixedHeader).y;
        }
    });

    return FreezeHeaders;
});