/*!
 * @module vcui.ui.LineChart
 * @license MIT License
 * @description LineChart 컴포넌트
 * @copyright VinylC UID Group
 * 
 * 
 */
define('ui/lineChart', ['jquery', 'vcui', 'libs/raphael.min'], function ($, core, Raphael) {
    "use strict";

    /**
     * @class
     * @description
     * @name
     * @extends vcui.ui.View
     */

    var LineChart = core.ui('LineChart', /** @lends vcui.ui.LineChart# */ {
        bindjQuery: 'lineChart',
        defaults: {
            //dataArr: [],
            //stageWidth: null,
            //stageHeight: null,
            sizeRadio: 0.53,
            margin: {
                top: 35,
                right: 40,
                bottom: 40,
                left: 40
            },
            pointSize: 7,
            isAnimate: false,
            isAverage: false,
            keyClasName : 'fl_l',
            valueClasName : 'fl_r',
            unit: "원" //mmss

        },
        selectors: {
        },

        initialize: function initialize(el, options) {
            var self = this;
            if (self.supr(el, options) === false) {
                return;
            }

            Raphael.el.addClass = function (className) {
                this.node.setAttribute("class", className);
                return this;
            };

            self._setting();
            self._build();
        },


        _setting: function _setting() {
            var self = this,
                opts = self.options;

            self.stageWidth = opts.stageWidth || self.$el.width();
            self.stageHeight = opts.stageHeight || self.stageWidth * opts.sizeRadio;
            self.margin = opts.margin;

            //console.log(self.$el.width(), self.stageWidth, self.stageHeight);

            self.$el.css('height',self.stageHeight);


            self.drawHeight = self.stageHeight - (self.margin.top + self.margin.bottom);
            self.drawWidth = self.stageWidth - (self.margin.left + self.margin.right);
            self.pointSize = opts.pointSize;
            self.isAverage = opts.isAverage;
            self.unit = opts.unit;

            //console.log(self.$target[0]);
            //self.$target = self.$target[0]? self.$target : $(opts.target);


            var dataArr = [];

            if (opts.target) {

                var obj, val;

                $(opts.target).children().each(function (idx, target) {

                    var key = $(target).find('.'+opts.keyClasName).text();
                    var keyHide = $(target).find('.'+opts.keyClasName+' .hide').text();
                    var value = $(target).find('.'+opts.valueClasName).text();
                    var valueHide = $(target).find('.'+opts.valueClasName + ' .hide').text();                    

                    if (keyHide) {
                        key = key.replace(keyHide, "");
                    }

                    if (valueHide) {
                        value = value.replace(valueHide, "");
                    }

                    if ($(target).hasClass('ui_average_txt')) {

                        self.averageValue = value;
                        return;
                    }

                    if (self.unit == "mmss") {
                        var vArr = value.replace("분", "/").split("/");
                        if (vArr.length > 1) {
                            val = core.number.parse(vArr[0]) * 60 + core.number.parse(vArr[1]);
                        }
                    } else {
                        val = core.number.parse(value);                        
                    }

                    var keyTilte =  $(target).find('.'+opts.keyClasName).data('title');
                    var valueTilte =  $(target).find('.'+opts.valueClasName).data('title');

                    obj = {
                        'title': keyTilte? keyTilte : key,
                        'label': valueTilte? valueTilte : value,
                        'value': val
                    };


                    dataArr.push(obj);
                });


            }


            self.dataArr = opts.dataArr || dataArr;

            self.aniArr = [];
            self.animArr = [];


        },

        _build: function _build() {

            var self = this;

            self.paper = Raphael(0, 0, self.stageWidth, self.stageHeight);
            self.paper.setViewBox(0, 0, self.stageWidth, self.stageHeight, false);
            self.$el.append(self.paper.canvas);

            var svg = $('svg');
            svg.css('position', 'static');
            svg.removeAttr('width');
            svg.removeAttr('height');

            self._createLine();
        },

        _createLine: function _createLine() {

            var self = this,
                opts = self.options;

            var minVal = parseFloat(Math.min.apply(Math, self.dataArr.map(function (o) {
                return o.value;
            })));
            var maxVal = parseFloat(Math.max.apply(Math, self.dataArr.map(function (o) {
                return o.value;
            })));

            var gRatio = self.drawHeight / (maxVal - minVal);
            var gDistance = self.drawWidth / (self.dataArr.length - 1);

            var pathStr = "";
            var pathFillStr = "";
            var stPathStr = "";
            var stPathFillStr = "";
            var fx, txt, xp, yp, obj;
            var ey = 0;
            var average = 0;

            var drawTop = self.drawHeight + self.margin.top;

            for (var i = 0; i < self.dataArr.length; i++) {

                obj = self.dataArr[i];
                average += parseFloat(obj.value);
                yp = (maxVal - parseFloat(obj.value)) * gRatio + self.margin.top;
                xp = gDistance * i + self.margin.left;
                txt = self.paper.text(xp, drawTop + 25, obj.title).attr({
                    "font-size": 12
                });
                txt.addClass("chart_title");

                if (xp <= self.margin.left) txt.translate(txt.getBBox().width / 2 - self.pointSize, 0);
                if (xp >= self.drawWidth + self.margin.left) txt.translate(txt.getBBox().width / 2 * -1 + self.pointSize, 0);

                if (i == 0) {

                    fx = xp;
                    pathStr += "M" + xp + " " + yp + " ";
                    stPathStr += "M" + xp + " " + drawTop + " ";
                } else if (i == self.dataArr.length - 1) {

                    pathFillStr = pathStr + "L" + xp + " " + yp + " L" + xp + " " + drawTop + " L" + fx + " " + drawTop + " z";
                    pathStr += "L" + xp + " " + yp + " M" + xp + " " + yp + "z";
                    stPathFillStr = stPathStr + "L" + xp + " " + drawTop + " L" + xp + " " + drawTop + " L" + fx + " " + drawTop + " z";
                    stPathStr += "L" + xp + " " + drawTop + " M" + xp + " " + drawTop + "z";
                    ey = yp;
                } else {

                    pathStr += "L" + xp + " " + yp + " ";
                    stPathStr += "L" + xp + " " + drawTop + " ";
                }

                var circle = null;
                var ballonObj = self._createBallon(obj.label, xp, yp, 'RB', self.drawWidth);

                if (!opts.isAnimate) {
                    ballonObj.target.translate(ballonObj.x, ballonObj.y);
                    circle = self.paper.circle(xp, yp, self.pointSize - 4).attr({
                        "fill": "#ffffff",
                        "stroke": "#c4343a",
                        "stroke-width": 2
                    });
                } else {
                    ballonObj.target.attr({
                        "opacity": 0
                    }).translate(ballonObj.x, drawTop);
                    circle = self.paper.circle(xp, drawTop, self.pointSize - 4).attr({
                        "fill": "#ffffff",
                        "stroke": "#c4343a",
                        "stroke-width": 2
                    });
                }

                circle.addClass("point_ball");
                self.aniArr.push({
                    target: circle,
                    param: {
                        cx: xp,
                        cy: yp
                    }
                });
                self.aniArr.push({
                    target: ballonObj.target,
                    param: {
                        "opacity": 1,
                        transform: "t" + ballonObj.x + "," + ballonObj.y
                    }
                });
            }

            average = average / self.dataArr.length;
            self.averageLine = (maxVal - average) * gRatio + self.margin.top;

            if (!opts.isAnimate) {
                stPathFillStr = pathFillStr;
                stPathStr = pathStr;
            }

            var pathFill = self.paper.path(stPathFillStr);
            pathFill.attr({
                "fill": "90-#ffffff-#ffe3d4:80-#ffe3d4",
                "stroke": 'none'
            });

            var path = self.paper.path(stPathStr);
            path.attr({
                "fill": 'none',
                "stroke": '#c4343a',
                "stroke-width": 2
            });

            path.toBack();
            pathFill.toBack();

            self.aniArr.push({
                target: path,
                param: {
                    path: pathStr
                }
            });
            self.aniArr.push({
                target: pathFill,
                param: {
                    path: pathFillStr
                }
            });


            if (self.isAverage && self.averageValue) {
                self._createAverage(average, ey);
            }

        },

        _createAverage: function _createAverage(average, ey) {

            var self = this,
                opts = self.options;

            var xp = self.stageWidth - self.margin.left + self.pointSize;
            var yp = self.averageLine;

            var stArr = ["M", self.margin.left - self.pointSize, yp];
            var edArr = ["M", self.margin.left - self.pointSize, yp, "L", xp, yp];
            if (ey && Math.abs(yp - ey) < 20) yp = yp - 20;

            var aBallonObj = self._createBallon(self.averageValue, xp, yp, 'RB', self.drawWidth, '#ffffff', '#333333', 'average_title');

            if (!opts.isAnimate) {
                stArr = edArr;
                aBallonObj.target.translate(aBallonObj.x, aBallonObj.y);
            } else {
                aBallonObj.target.attr({
                    "opacity": 0
                }).translate(aBallonObj.x, aBallonObj.y + 20);
            }

            var averagePath = self.paper.path(stArr).attr({
                "fill": 'none',
                "stroke": '#202020',
                "stroke-dasharray": ".",
                "stroke-width": 2
            });

            self.aniArr.push({
                target: averagePath,
                param: {
                    path: edArr
                },
                duration: 1000,
                delay: 1000
            });
            self.aniArr.push({
                target: aBallonObj.target,
                param: {
                    "opacity": 1,
                    transform: "t" + aBallonObj.x + "," + aBallonObj.y
                },
                duration: 300,
                delay: 1700
            });
        },

        goAnimate: function goAnimate() {

            var self = this,
                opts = self.options;

            if (!opts.isAnimate) return;

            for (var i = 0; i < self.aniArr.length; i++) {
                var obj = self.aniArr[i];
                if (obj.delay) {
                    var anim = Raphael.animation(obj.param, obj.duration, '<>');
                    obj["target"].animate(anim.delay(obj.delay));
                    self.animArr.push(anim);
                } else {
                    obj["target"].animate(obj["param"], 1000, '<>');
                }
            }

        },


        clear: function clear() {

            var self = this;

            for (var i = 0; i < self.aniArr.length; i++) {
                self.aniArr[i]["target"].stop();
            }

            for (var i = 0; i < self.animArr.length; i++) {
                self.animArr[i].stop();
            }

            self.aniArr = [];
            self.animArr = [];
            self.paper.clear();
        },
        /**
         * 이벤트 바인딩
         * @private
         */
        _bindEvents: function _bindEvents() {
            var self = this;
        },

        _createBallon: function _createBallon(txt, x, y, direct, limitWidth, textColor, ballonColor, classNm) {

            var self = this;
            var dt = direct.toLowerCase();
            var bgColor = ballonColor ? ballonColor : 'none';
            var txtColor = textColor ? textColor : '#000000';

            var group = self.paper.set();
            var text = self.paper.text(0, 0, txt);
            text.attr({
                "fill": txtColor,
                "font-size": 14
            });
            text.addClass(classNm ? classNm : "ballon_title");
            var ballonW = text.getBBox().width + 20;
            text.attr({
                "x": ballonW / 2,
                "y": 15
            });

            var rect = self.paper.rect(0, 0, ballonW, 30, 5).attr({
                "fill": bgColor,
                "stroke": 'none'
            });

            var p = ["M", 0, 0, "L", 0, 0, "L", 0, 0, "Z"];
            var bx, by;
            var xpos, ypos;

            if (dt == "rb") {

                bx = ballonW - 25, xpos = x - bx, by = 29;
                ypos = ballonColor ? y - 50 : y - 40;
                p = ["M", bx, by, "L", bx + 10, by, "L", bx, by + 10, "Z"];
            } else if (dt == "lb") {

                bx = 15, xpos = x - bx - 10, by = 29;
                ypos = ballonColor ? y - 50 : y - 40;
                p = ["M", bx, by, "L", bx + 10, by, "L", bx + 10, by + 10, "Z"];
            } else if (dt == "rt") {

                bx = ballonW - 25, xpos = x - bx, by = -9;
                ypos = ballonColor ? y + 20 : y + 10;
                p = ["M", bx, by, "L", bx + 10, by, "L", bx, by - 10, "Z"];
            } else if (dt == "lt") {
                bx = 15, xpos = x - bx - 10, by = -9;
                ypos = ballonColor ? y + 20 : y + 10;
                p = ["M", bx, by, "L", bx + 10, by, "L", bx + 10, by - 10, "Z"];
            }

            if (limitWidth && xpos + ballonW > limitWidth) {
                if (dt == 'lb' || dt == 'rb') {
                    bx = ballonW - 13;
                    xpos = ballonColor ? x - ballonW : x - ballonW + 10;
                    p = ["M", bx, by, "L", bx + 10, by, "L", bx + 10, by + 10, "Z"];
                } else if (dt == 'lt' || dt == 'rt') {
                    bx = ballonW - 13;
                    xpos = ballonColor ? x - ballonW : x - ballonW + 10;
                    p = ["M", bx, by, "L", bx + 10, by, "L", bx + 10, by - 10, "Z"];
                }
            }

            if (xpos < 0) {

                if (dt == 'lb' || dt == 'rb') {
                    bx = 3;
                    xpos = ballonColor ? x : x - 15;
                    p = ["M", bx, by, "L", bx + 10, by, "L", bx, by + 10, "Z"];
                } else if (dt == 'lt' || dt == 'rt') {
                    bx = 3;
                    xpos = ballonColor ? x : x - 15;
                    p = ["M", bx, by, "L", bx + 10, by, "L", bx, by - 10, "Z"];
                }
            }

            var arrow = self.paper.path(p).attr({
                "fill": bgColor,
                "stroke": "none"
            });
            text.toFront();
            group.push(text, arrow, rect);
            return {
                target: group,
                x: xpos,
                y: ypos
            };
        }

    });
    ///////////////////////////////////////////////////////////////////////////////////////

    return LineChart;
});