/*!
 * @module vcui.ui.PieChart
 * @license MIT License
 * @description PieChart 컴포넌트
 * @copyright VinylC UID Group
 * 
 * 
 */
define('ui/pieChart', ['jquery', 'vcui', 'libs/fraphael'], function ($, core, Raphael) {
    "use strict";

    /**
     * @class
     * @description
     * @name
     * @extends vcui.ui.View
     */

    var PieChart = core.ui('PieChart', /** @lends vcui.ui.PieChart# */ {
        bindjQuery: 'pieChart',
        defaults: {
            //dataArr: [],
            //stageWidth: null,
            //stageHeight: null,
            sizeRadio: 0.9,
            firstAngle: 30,
            margin: {
                top: 25,
                right: 20,
                bottom: 15,
                left: 10
            },
            isAnimate: false,
            keyClasName: 'item_th',
            valueClasName: 'item_td'


        },
        selectors: {},

        initialize: function initialize(el, options) {
            var self = this;
            if (self.supr(el, options) === false) {
                return;
            }

            Raphael.el.addClass = function (className) {
                this.node.setAttribute("class", className);
                return this;
            };

            self._setting();
            self._build();
            
        },


        _setting: function _setting() {
            var self = this,
                opts = self.options;

            self.stageWidth = opts.stageWidth || self.$el.width();
            //self.stageHeight = opts.stageHeight || self.$el.height();
            self.stageHeight = opts.stageHeight || self.stageWidth * opts.sizeRadio;
            self.margin = opts.margin;
            self.drawHeight = self.stageHeight - (self.margin.top + self.margin.bottom);
            self.drawWidth = self.stageWidth - (self.margin.left + self.margin.right);

            //self.$el.css('height', self.stageHeight);


            var pieCx = self.drawWidth / 2;
            var pieCy = self.drawHeight / 2;

            self.pieR = Math.min(pieCx, pieCy);
            self.pieCx = pieCx + self.margin.left;
            self.pieCy = pieCy + self.margin.top;
            self.fristAngle = opts.firstAngle;

            self.gapWidth = (self.drawWidth - self.pieR * 2) / 2;

            var dataArr = [];

            if (opts.target) {

                var obj, val;
                var total = 0;

                $(opts.target).children().each(function (idx, target) {

                    /*
                    <span class="item_th"><mark class="color color_1"></mark>이동전화</span>
                    <span class="item_td"><em>22,100</em>원</span>
                    */

                    var key = $(target).find('.' + opts.keyClasName).text();
                    var keyHide = $(target).find('.' + opts.keyClasName + ' .hide').text();
                    var value = $(target).find('.' + opts.valueClasName).text();
                    var valueHide = $(target).find('.' + opts.valueClasName + ' .hide').text();
                    var color = $(target).find('.color').css('background-color');

                    if (keyHide) {
                        key = key.replace(keyHide, "");
                    }

                    if (valueHide) {
                        value = value.replace(valueHide, "");
                    }

                    val = core.number.parse(value);

                    if (val > 0) {
                        total += val;
                        obj = {
                            'title': key,
                            'value': val,
                            'color': color
                        };
                        dataArr.push(obj);
                    }

                });

                // value 기준으로 정렬
                dataArr.sort(function (a, b) {
                    if (a.value > b.value) {
                        return -1;
                    }
                    if (a.value < b.value) {
                        return 1;
                    }
                    return 0;
                });


                for (var i = 0; i < dataArr.length; i++) {
                    obj = dataArr[i];
                    val = (obj.value / total * 100).toFixed(1);
                    obj.label = val + "%";
                }
            }

            self.dataArr = opts.dataArr || dataArr;
            self.aniArr = [];
            self.animArr = [];

        },

        _build: function _build() {

            var self = this;


            self.paper = Raphael(0, 0, '100%', '100%');


            self.paper.customAttributes.arc = function (xloc, yloc, startAngle, endAngle, R, flag) {

                var x1 = xloc + R * Math.cos(-startAngle * Math.PI / 180),
                    x2 = xloc + R * Math.cos(-endAngle * Math.PI / 180),
                    y1 = yloc + R * Math.sin(-startAngle * Math.PI / 180),
                    y2 = yloc + R * Math.sin(-endAngle * Math.PI / 180)


                if (endAngle - startAngle == 360) {
                    return {
                        path: [["M", xloc, yloc + R], ["A", R, R, 0, 1, 0, xloc - 0.001, yloc + R]]
                    };
                }

                return {
                    path: [["M", x1, y1], ["A", R, R, 0, +(endAngle - startAngle > 180), 0, x2, y2]]
                };
            };


            self.paper.setViewBox(0, 0, self.stageWidth, self.stageHeight, false);
            self.$el.append(self.paper.canvas);


            var nx = 0;
            var radio = 1;
            var ptObj = self._getPiePos(self.dataArr, self.pieCx, self.pieCy, self.pieR, self.fristAngle);


            if (ptObj.isEtc) {

                radio = 0.9;
                nx = -(self.drawWidth - self.pieR * 2) / 2 - self.pieR * (1 - radio) + 10;

                for (var i = 0; i < ptObj.outArr.length; i++) {

                    ptObj.outArr[i].r = ptObj.outArr[i].r * radio;
                    ptObj.outArr[i].tx = ptObj.outArr[i].tx * radio;
                    ptObj.outArr[i].ty = ptObj.outArr[i].ty * radio;
                }
                self._createEtc(ptObj.outArr, nx, 0);
            }


            self._createPie(ptObj.outArr, nx, 0);
            self._createLabel(ptObj.outArr, nx, 0, 1);

            var bg = self.paper.circle(self.pieCx + nx, self.pieCy, self.pieR * radio).attr({
                fill: "#ffffff",
                stroke: "none",
                opacity: 0
            }).shadow(0, 0, 6, 0.4);

            bg.toBack();
            self.aniArr.push({target: bg, delay: 800, duration: 500, param: {opacity: 1}});


        },

        _getPiePos: function _getPiePos(arr, cx, cy, r, stAngle) {
            var self = this;

            var angle = stAngle ? stAngle : 0;
            var endAngle = angle;
            var start = 0;
            var total = 0;
            var outObj = {};
            var isEtc = false;
            var fSize;
            var outArr = [];

            for (var i = 0; i < arr.length; i++) {
                total += parseFloat(arr[i]["value"]);
            }

            var obj, value;
            var angleplus, popangle, delta, txt, txtBox, txtBg;

            for (var i = 0; i < arr.length; i++) {

                obj = arr[i];
                value = parseFloat(obj.value);
                angleplus = 360 * value / total;
                popangle = angle + (angleplus / 2);
                fSize = (angleplus - 60) * 0.2 + 12;

                //console.log(fSize, angleplus);

                delta = r / 2 * -0.6 - (fSize - 15) * 1.2;
                //delta = r/2*(angleplus*-0.01);
                //delta = r/2*-0.1;


                outArr.push({
                    title: obj.title,
                    label: obj.label,
                    value: value,
                    cx: cx,
                    cy: cy,
                    r: r,
                    delta: delta,
                    tx: cx + (r + delta) * Math.cos(-popangle * Math.PI / 180),
                    ty: cy + (r + delta) * Math.sin(-popangle * Math.PI / 180),
                    startAngle: angle,
                    endAngle: angle + angleplus,
                    popangle: popangle,
                    fontSize: fSize,
                    color: obj.color,
                    bcolor: obj.color,
                    isVisible: angleplus * r * 0.01 < 25 ? false : true

                });


                if (angleplus * r * 0.01 < 25) {
                    isEtc = true;
                } else {
                    endAngle += angleplus;
                }

                angle += angleplus;
                start += .1;
            }


            outObj.isEtc = isEtc;
            outObj.outArr = outArr;
            outObj.endAngle = endAngle;

            return outObj;

        },

        _createSector: function _createSector(cx, cy, r, startAngle, endAngle, params) {
            var self = this;

            var x1 = cx + r * Math.cos(-startAngle * Math.PI / 180),
                x2 = cx + r * Math.cos(-endAngle * Math.PI / 180),
                y1 = cy + r * Math.sin(-startAngle * Math.PI / 180),
                y2 = cy + r * Math.sin(-endAngle * Math.PI / 180);

            return ["M", x1, y1, "A", r, r, 0, +(endAngle - startAngle > 180), 0, x2, y2];


        },

        _createLabel: function _createLabel(arr, xp, yp, radio) {

            var self = this;
            var obj, txt, txtBox, txtBg, txtColor, tx, ty, pTxt;
            var xpos = xp ? xp : 0;
            var ypos = yp ? yp : 0;


            var opacity = self.options.isAnimate ? 0 : 1;
            var fSize;

            for (var i = 0; i < arr.length; i++) {
                obj = arr[i];
                if (obj.isVisible) {

                    fSize = obj.fontSize;

                    //console.log(fSize);
                    if (fSize < 10) fSize = 10;
                    if (fSize > 16) fSize = 16;

                    tx = obj.cx + (obj.r * radio + obj.delta) * Math.cos(-obj.popangle * Math.PI / 180);
                    ty = obj.cy + (obj.r * radio + obj.delta) * Math.sin(-obj.popangle * Math.PI / 180);


                    if (arr.length == 1) {
                        txt = self.paper.text(self.pieCx, self.pieCy - 25, obj.title)
                            .attr({fill: "#ffffff", stroke: "none", opacity: opacity, "font-size": 20});

                        pTxt = self.paper.text(self.pieCx, self.pieCx - 5, obj.label)
                            .attr({fill: "#ffffff", stroke: "none", opacity: opacity, "font-size": 34});

                    } else {
                        txt = self.paper.text(tx + xpos, ty + ypos - fSize / 1.3, obj.title)
                            .attr({fill: "#ffffff", stroke: "none", opacity: opacity, "font-size": fSize});
                        //.shadow(2, 2, 2, 0.3);

                        pTxt = self.paper.text(tx + xpos, ty + ypos + fSize / 1.3, obj.label)
                            .attr({fill: "#ffffff", stroke: "none", opacity: opacity, "font-size": fSize * 1.5});
                        //.shadow(2, 2, 2, 0.3);
                    }


                    txt.toFront();
                    pTxt.toFront();
                    self.aniArr.push({target: txt, delay: obj.popangle * 1.2, duration: 300, param: {opacity: 1}});
                    self.aniArr.push({target: pTxt, delay: obj.popangle * 1.2, duration: 300, param: {opacity: 1}});

                }
            }

        },

        _createPie: function _createPie(arr, xp, yp, stroke) {

            var self = this;
            var obj, p, pathStr, toPathStr;
            var param = {};
            var stroke = stroke ? stroke : "#fff";
            var xpos = xp ? xp : 0;
            var ypos = yp ? yp : 0;

            var delayTime = 0;
            var duration = 0;

            for (var i = 0; i < arr.length; i++) {
                obj = arr[i];

                if (self.options.isAnimate) {
                    param = {
                        stroke: obj.bcolor,
                        "stroke-width": obj.r,
                        arc: [obj.cx + xpos, obj.cy + ypos, obj.startAngle, obj.startAngle, obj.r / 2],
                        opacity: 0
                    };
                } else {
                    param = {
                        stroke: obj.bcolor,
                        "stroke-width": obj.r,
                        arc: [obj.cx + xpos, obj.cy + ypos, obj.startAngle, obj.endAngle, obj.r / 2]
                    };

                }

                if (arr.length == 1) {

                    param.arc.push(true);

                    p = self.paper.path().attr(param);
                } else {
                    p = self.paper.path().attr(param);
                }
                p.toBack();

                duration = parseInt((obj.endAngle - obj.startAngle));
                self.aniArr.push({
                    target: p,
                    delay: parseInt(delayTime * 0.95),
                    duration: duration,
                    param: {opacity: 1, arc: [obj.cx + xpos, obj.cy + ypos, obj.startAngle, obj.endAngle, obj.r / 2]}
                });

                delayTime += duration;
            }
        },


        _createEtc: function _createEtc(arr, dx, dy) {

            var self = this;
            var itemH = 20;
            var xpos = self.stageWidth - self.margin.right;
            var ypos = self.margin.top;
            var dXpos = dx ? dx : 0;
            var dYpos = dy ? dy : 0;

            var tArr = [];

            for (var i = 0; i < arr.length; i++) {
                if (!arr[i].isVisible) {
                    tArr.push(arr[i]);
                }
            }

            tArr.reverse();


            var line, txt, sx, sy, gx, gx2;


            for (var i = 0; i < tArr.length; i++) {

                ypos += itemH;

                sx = tArr[i].cx + (tArr[i].r) * Math.cos(((tArr[i].endAngle - tArr[i].startAngle) / 2 + tArr[i].startAngle) * Math.PI / 180) + dXpos;
                sy = tArr[i].cy + (tArr[i].r) * Math.sin(-((tArr[i].endAngle - tArr[i].startAngle) / 2 + tArr[i].startAngle) * Math.PI / 180) + dYpos;

                txt = self.paper.text(xpos, ypos, tArr[i]["title"] + " " + tArr[i]["label"])
                    .attr({
                        "fill": "#000000",
                        "opacity": 0,
                        "stroke": "none",
                        "font-weight": "bold",
                        "font-size": 12,
                        "font-family": 'Helvetica'
                    });

                txt.attr({x: xpos - txt.getBBox().width / 2});

                gx = xpos - txt.getBBox().width - 15;
                gx2 = gx + 10;
                if (gx < sx) {
                    gx = gx2;
                    gx2 = gx + 2;
                }

                line = self.paper.path("M" + sx + " " + sy + " L" + gx + " " + ypos + " L" + gx2 + " " + ypos + " M" + sx + " " + sy + " z")
                    .attr({"fill": "none", "stroke": tArr[i].bcolor, "stroke-width": 1, "opacity": 0});

                self.aniArr.push({target: txt, delay: i * 100 + 200, duration: 300, param: {opacity: 1}});
                self.aniArr.push({target: line, delay: i * 100 + 200, duration: 300, param: {opacity: 1}});

            }


        },

        goAnimate: function goAnimate() {

            var self = this,
                opts = self.options;

            if (!opts.isAnimate) return;

            for (var i = 0; i < self.aniArr.length; i++) {
                var obj = self.aniArr[i];
                if (obj.delay) {
                    var anim = Raphael.animation(obj.param, obj.duration, 'linear');
                    obj["target"].animate(anim.delay(obj.delay));
                    self.animArr.push(anim);
                } else {
                    obj["target"].animate(obj["param"], obj.duration, 'linear');
                }
            }
        },


        clear: function clear() {

            var self = this;

            for (var i = 0; i < self.aniArr.length; i++) {
                self.aniArr[i]["target"].stop();
            }

            for (var i = 0; i < self.animArr.length; i++) {
                self.animArr[i].stop();
            }

            self.aniArr = [];
            self.animArr = [];
            self.paper.clear();
        },
        /**
         * 이벤트 바인딩
         * @private
         */
        _bindEvents: function _bindEvents() {

            var self = this;
        }

    });
    ///////////////////////////////////////////////////////////////////////////////////////

    return PieChart;
});